import unittest

import os
import csv
from io import StringIO
from pyslr import BibImporter,Format


bibFile = r"""
    @inproceedings{2018-Back-ESOCC-FaasBenchmarking,
        author     = "{B}ack, {T}imon and {A}ndrikopoulos, {V}asilios",
        title      = "{U}sing a {M}icrobenchmark to {C}ompare {F}unction as a {S}ervice {S}olutions",
        booktitle  = "{P}roceedings of the 7th {E}uropean {C}onference on {S}ervice-{O}riented and {C}loud {C}omputing",
        series     = "{ESOCC}'18",
        year       = "2018",
        month      = "{A}ug",
        isbn       = "978-3-319-99819-0",
        location   = "{C}omo, {I}taly",
        pages      = "146--160",
        numpages   = "15",
        doi        = "https://doi.org/10.1007/978-3-319-99819-0_11",
        editor     = "{K}ritikos, {K}yriakos and {P}lebani, {P}ierluigi  and de {P}aoli, {F}lavio",
        publisher  = "{S}pringer {I}nternational {P}ublishing",
        address    = "{C}ham"
    }

    @article{2019-Hellerstein-CoRR-ServerlessComputing,
        author    = "{J}oseph {M}. {H}ellerstein and
                    {J}ose {M}. {F}aleiro and
                    {J}oseph {E}. {G}onzalez and
                    {J}ohann {S}chleier{-}{S}mith and
                    {V}ikram {S}reekanti and
                    {A}lexey {T}umanov and
                    {C}henggang {W}u",
        title     = "{S}erverless {C}omputing: {O}ne {S}tep {F}orward, {T}wo {S}teps {B}ack",
        journal   = "{C}o{RR}",
        volume    = {abs/1812.03651},
        year      = "2018",
        month     = "{J}an",
        day       = "01",
        pages     = "1-8",
        numpages  = "8",
        publisher = "arXiv",
        url       = {http://arxiv.org/abs/1812.03651}
    }

    @phdthesis{2018-Klems-Phd-ExperimentDrivenEvaluation,
        author    = "Klems, Markus",
        title     = "Experiment-driven Evaluation of Cloud-based Distributed Systems",
        school    = "TU Berlin",
        year      = "2016"
    }


    %%% Books
    @book{2017-Bermbach-Book-CloudServiceBenchmarking,
        author    = "Bermbach, David and Wittern, Erik and Tai, Stefan",
        title     = "Cloud Service Benchmarking: Measuring Quality of Cloud Services from a Client Perspective",
        year      = "2017",
        publisher = "Springer International Publishing",
        address   = "Cham",
        isbn      = "978-3-319-55482-2"
    }

"""


class BibImporterTest(unittest.TestCase):


    def test_upper(self):
        output = StringIO()
        writer = csv.writer(output, delimiter=',',quotechar='"', quoting=csv.QUOTE_MINIMAL,strict=True)

        fmt = Format(writer)

        bib = BibImporter()
        bib.parse_and_write(StringIO(bibFile),fmt)

        assert(len(output.getvalue()) > 0)

        publications = fmt.readAsPublications(StringIO(output.getvalue()))

        types = list(map(lambda x:x.type,publications))

        assert("book" in types)
        assert("article" in types)
        assert("inproceedings" in types)
        assert("phdthesis" in types)

       

if __name__ == '__main__':
    unittest.main()


import csv
import hashlib
import logging
import pandas as pd
from datetime import datetime

__all__ = ["Format","Publication"]

logger = logging.getLogger(__name__)




class Publication(object):

    def __init__(self,id=None, peer_reviewed=None, publisher=None, source=None, search_date=None, 
    title=None, year=None, authors=None, type=None, country=None, school=None, book_title=None, 
    series=None, num_pages=None, doi=None, journal_name=None, volume=None, keywords=None, 
    abstract=None, tried=None, references=None, isbn=None):
        self.id=id
        self.peer_reviewed=peer_reviewed
        self.publisher=publisher
        self.source=source
        self.search_date=search_date
        self.title=title
        self.year=year
        self.authors=authors
        self.type=type
        self.country=country
        self.school=school
        self.book_title=book_title
        self.series=series
        self.num_pages=num_pages
        self.doi=doi
        self.journal_name=journal_name
        self.volume=volume
        self.keywords=keywords
        self.abstract=abstract
        self.tried=tried
        self.references=references
        self.isbn=isbn
    
    def __ensure_not_none(s):
        if s is None:
            return ""
        return s

    def _reset(self):
        self.id = None
        self.peer_reviewed = None
        self.publisher = None
        self.source = None
        self.search_date = None
        self.title = None
        self.year = None
        self.authors = None
        self.type = None
        self.country = None
        self.school = None
        self.book_title = None
        self.series = None
        self.num_pages = None
        self.doi = None
        self.journal_name = None
        self.volume = None
        self.keywords = None
        self.abstract = None
        self.tried = None
        self.references = None
        self.isbn = None        
 
    @staticmethod
    def Paper(peer_reviewed,publisher,source,title,year,authors=[],
                country="",book_title="",series="",num_pages=None,doi=None,
                keywords=[],abstract=None,tried=None,references=None):
        """
            Builder method to create a paper publication
        """
        return Publication().fromPaper(peer_reviewed,publisher,source,title,year,
        authors,country,book_title,series,num_pages,doi,keywords,
        abstract,tried,references)  

    def fromPaper(self,peer_reviewed,publisher,source,title,year,authors=[],
                country="",book_title="",series="",num_pages=None,doi=None,
                keywords=[],abstract=None,tried=None,references=None):
        self._reset()
        self.id = Format.generate_id(title,authors,year,"inproceedings")
        self.peer_reviewed = peer_reviewed
        self.publisher = publisher
        self.source = source
        self.search_date = datetime.now()
        self.title = title
        self.year = year
        self.authors = authors
        self.type = "inproceedings"
        self.country = country
        self.book_title = book_title
        self.series = series
        self.num_pages = num_pages
        self.doi = doi
        self.keywords = keywords
        self.abstract = abstract
        self.tried = tried
        self.references = references
        return self

    @staticmethod
    def Journal(peer_reviewed,publisher,source,title,year,authors=[],
                country="",journal_name="",volume="",num_pages=None,doi=None,
                keywords=[],abstract=None,tried=None,references=None):
        """
            Builder method to create a journal publication
        """
        return Publication().fromJournal(peer_reviewed,publisher,source,title,year,
        authors,country,journal_name,volume,num_pages,doi,keywords,
        abstract,tried,references)
        
    def fromJournal(self,peer_reviewed,publisher,source,title,year,authors=[],
                country="",journal_name="",volume="",num_pages=None,doi=None,
                keywords=[],abstract=None,tried=None,references=None):
        self._reset()
        self.id = Format.generate_id(title,authors,year,"article")
        self.peer_reviewed = peer_reviewed
        self.publisher = publisher
        self.source = source
        self.search_date = datetime.now()
        self.title = title
        self.year = year
        self.authors = authors
        self.type = "article"
        self.country = country
        self.num_pages = num_pages
        self.doi = doi
        self.journal_name = journal_name
        self.volume = volume
        self.keywords = keywords
        self.abstract = abstract
        self.tried = tried
        self.references = references
        return self

    @staticmethod
    def Book(peer_reviewed,publisher,source,title,year,authors=[],
                num_pages=None,isbn=None,doi=None,references=None,tried=None):
        """
            Builder method to create a book publication
        """
        return Publication().fromBook(peer_reviewed,publisher,source,title,year,authors,
                num_pages,isbn,doi,references,tried)

    def fromBook(self,peer_reviewed,publisher,source,title,year,authors=[],
                num_pages=None,isbn=None,doi=None,references=None,tried=None):
        self._reset()
        self.id = Format.generate_id(title,authors,year,"book")
        self.peer_reviewed = peer_reviewed
        self.publisher = publisher
        self.source = source
        self.search_date = datetime.now()
        self.title = title
        self.year = year
        self.authors = authors
        self.type = "book"
        self.num_pages = num_pages
        self.doi = doi
        self.references = references
        self.tried = tried
        self.isbn = isbn
        return self

    @staticmethod
    def PHD(peer_reviewed,publisher,source,title,year,authors=[],
                country="",school=None,num_pages=None,doi=None,
                abstract=None,isbn=None,references=None,tried=None):
        """
            Build method for phd thesis publications
        """
        return Publication().fromPHD(peer_reviewed,publisher,source,title,year,authors[0],
                country,school,num_pages,doi,abstract,isbn,references,tried)

    def fromPHD(self,peer_reviewed,publisher,source,title,year,author=None,
                country="",school=None,num_pages=None,doi=None,
                abstract=None,isbn=None,references=None,tried=None):
        self._reset()
        self.id = Format.generate_id(title,[author],year,"phdthesis")
        self.peer_reviewed = peer_reviewed
        self.publisher = publisher
        self.source = source
        self.search_date = datetime.now()
        self.title = title
        self.year = year
        self.authors = [author]
        self.type = "phdthesis"
        self.country = country
        self.school = school
        self.num_pages = num_pages
        self.doi = doi
        self.abstract = abstract
        self.references = references
        self.isbn = isbn
        self.tried = tried
        return self

    def toWritableList(self):
        return list(map(Publication.__ensure_not_none,[
            self.id,
            self.peer_reviewed,
            self.publisher,
            self.source,
            None if self.search_date is None else Format.format_date(self.search_date),
            self.title,
            self.year,
            self.authors,
            self.type,
            self.country,
            self.school,
            self.book_title,
            self.series,
            self.num_pages,
            self.doi,
            self.journal_name,
            self.volume,
            self.keywords,
            self.abstract,
            self.tried,
            self.references,
            self.isbn
        ]))
    
    def fromList(args):
        pub = Publication(*args)
        pub.search_date = datetime.strptime(pub.search_date,"%Y-%m-%dT%H:%M")
        return pub


    def __str__(self):
        return str(self.toWritableList())


class Format():
    _version = "v1"
    _v1_header = ["id","peer_reviewed","publisher","source","search_date","title","year","authors","type","country","school","book_title","series","num_pages","doi","journal_name","volume","keywords","abstract","tried","references","isbn"]

    def __init__(self,writer=None):
        self.writer = writer
        super().__init__()

    def readAsPanda(self, fname):
        return pd.read_csv(fname,converters=
                    {"keywords": Format.csv_list_reader,
                     "authors":Format.author_list_reader,
                     "tried":Format.csv_list_reader,
                     "references":Format.csv_list_reader
                    })
    
    def readAsPublications(self,fname):
        reader = csv.reader(fname)

        return map(Publication.fromList,reader)

    @staticmethod
    def openWriter(file, writeHeader=True):
        return Format().openWriter(filename,writeHeader)

    def openWriter(self,file, writeHeader=True):
        writer =  csv.writer(file, delimiter=',',quotechar='"', quoting=csv.QUOTE_MINIMAL,strict=True)
        if writeHeader:
            writer.writerow(self._v1_header)

        self.writer = writer

        return writer
    
    @staticmethod
    def format_date(date):
        return date.strftime("%Y-%m-%dT%H:%M")
    
    @staticmethod
    def generate_id(title,authros,year,publication_type):
        key = title+str(authros)+str(year)+publication_type

        return hashlib.sha1(key.encode('utf-8')).hexdigest()
    
    @staticmethod
    def writeRow(writer,publication):
        Format(writer).addRow(publication)

    def addRow(self,publication):
        self.writer.writerow(publication.toWritableList())

    @staticmethod
    def write(writer,publications=[]):
        Format(writer).writeAll(publications)

    def writeAll(self,writer,publications=[]):
        for publication in publications:
            self.writeRow(publication)

    @staticmethod
    def csv_list_reader(line):
        #remove [] and split by ,
        string_list = line.strip("[]").split(",")
        #remove the '' around each string and convert to list
        return list(map(lambda y:y.strip("''"),string_list))

    @staticmethod
    def author_list_reader(line):
        #remove [] and split by ','
        string_list = line.strip("[]").split("','")
        #clean up elements
        elements = map(lambda elem:elem.replace("'","").strip("()"),string_list)
        #split up author strings, and convert to tuple
        author_pairs = map(lambda authors:tuple(map(lambda q:q.strip(),authors.split(","))),elements)
        return list(author_pairs)



from pyslr.find import *
from pyslr.common import Format

class Search():
    def __init__(self,modules=[]):
        self.modules=modules
    
    def serach(self,output,keywords=[]):
        slr = Format()
        with open(output,"w") as out:
            writer = slr.openWriter(out)

            for module in self.modules:
                find = module()
                print("using {}".format(find.name()))

                results = find.search(keywords)

                for result in results:
                    slr.addRow(result)
        
        




import scrapy
from urllib.parse import urlencode
from scrapy.crawler import CrawlerProcess
from pyslr.find import Finder
from pyslr.common import Publication
import requests
import json
import time


class ACMSeachDOISpider(scrapy.Spider):
    name="ACM-Serach-Spider"
    results_per_page=50
    def __init__(self, after_year=2015,query=None,results=[], *args, **kwargs):
        super(ACMSeachDOISpider, self).__init__(*args, **kwargs)
        q = {'AllField':query}
        allfields=urlencode(q).replace('%28','(').replace("%22","\"").replace("%29",")") 
        baseurl = 'https://dl.acm.org/action/doSearch?fillQuickSearch=false&expand=dl&AfterYear={}&{}++'
        self.start_urls = [baseurl.format(after_year,allfields)]
        self.results = results
        print("using",self.start_urls)

    def parse(self,response):

        for doi in map(lambda x:x[len('https://doi.org/'):],response.xpath('//a[contains(@class,"item__doi")]/@href').getall()):
            self.results.append(doi)
            yield {"doi":doi}
        
        baseurl =response.request.url
        if "startPage" not in baseurl:
            
            #number of found items/items per page
            pages = round(int(response.css('span.hitsLength::text').get().replace(',',''))/self.results_per_page)

            for page in range(pages):
                yield scrapy.Request('{}&pageSize={}&startPage={}'.format(baseurl,self.results_per_page,page))



class ACMSerach(Finder):
    def name(self):
        return "ACM"

    def from_dois(self,filename):
        dois = []
        with open(filename,"r") as f:
            dois = list(map(lambda x:x.strip(),f.readlines()))
        
        self._fromDoiToInternal(dois,search_name="FROM_DOIS")

    def search(self,keywords=[]):
        query = []
        for keyword in keywords:
            query.append("Abstract:({})".format(keyword))
        query=" OR ".join(query)
        return self.search_raw(query)

    def _fromDoiToInternal(self,dois,search_name):
        items = self._optainDetails(dois)
        
        with open('results.txt',"w+") as f:
            json.dump(items,f)
        
        return self._convertACMDetails(items,search_name)
    

    def from_jsonDump(self,filename):
        with open(filename,"r") as f:
            items = json.load(f)
        return self._convertACMDetails(items,search_name="FROM DUMP")

    def _convertACMDetails(self,items,search_name="DEBUG"):
        converter = {
            'ARTICLE_JOURNAL':self._asJournal, 
            'PAPER_CONFERENCE':self._asPaper
        }

        search_results = []
        failed = []
        
        for item in items:
            try:
                item = self._unpack(item)
                if 'type' not in item:
                    print("don't know hat this is: {}".format(item))
                    continue
                if item['type'] in converter:
                    try:
                        search_results.append(converter[item['type']](item,search_name))
                    except Exception as e:
                        print("failed to add {} due to {}".format(item['id'],e))
                        failed.append(item)
                else:
                    print("failed to add {}".format(item['DOI']))
                    failed.append(item)
            except Exception as err:
                print("something went wrong, its my fault \_o_/. {}".format(err))
    
        with open('failed_bibs.txt',"w+") as f:
            json.dump(failed,f)

        return search_results


    def search_raw(self,query):
        search_name = "{}-{}".format(self.name(),hash(query)%10**8)
        
        results = self._searchDoisUsingSpyder(query)

        print("using {} we collected  {} dois, getting detailed information now...".format(query,len(results)))
        
        self._storeIntermediateResults('dois.txt',results,"w")

        return self._fromDoiToInternal(results,search_name)

    def _optainDetails(self,results):
        items = []
        for doi_chunk in chunks(results,100):
            doi_response = requests.post('https://dl.acm.org/action/exportCiteProcCitation',data={'targetFile':'custom-bibtex','format':'bibTex','dois':",".join(doi_chunk)}) 
            if doi_response.status_code == 200:
                data = json.loads(doi_response.content)
                #the bitTex are in the imtes
                items = items+data['items']
            else:
                print("could not collect dois, cause:",doi_response.status_code,doi_response.content)
                self._storeIntermediateResults('failed.txt',doi_chunk,"a+")
            time.sleep(5)
        
        return items



    def _storeIntermediateResults(self, filename,results,mode="w+"):
        with open(filename,mode) as f:
            f.write("\n".join(results))

    def _searchDoisUsingSpyder(self, query):
        process = CrawlerProcess()
        process.settings.set('DOWNLOAD_DELAY',  5, priority='cmdline')
        process.settings.set('COOKIES_DEBUG',  True, priority='cmdline')

        results=[]

        process.crawl(ACMSeachDOISpider,query=query,results=results)
        process.start()
        return results
    
    def _unpack(self,item):
        return list(item.items())[0][1]

    def _author(self,authors): 
        result = []
        for x in authors:
            if 'family' in x and 'given' in x:
                result.append("({},{})".format(x['family'],x['given']))
            elif 'family' in x:
                result.append("({},)".format(x['family']))
        return result
    
    def _asPaper(self,item,src):
        keywords = []
        if "keyword" in item:
            keywords = item['keyword'].split(',')
        return Publication.Paper(peer_reviewed=True,publisher="ACM",title=item['title'],num_pages=item['number-of-pages'],year=item['original-date']['date-parts'][0][0],country=item['event-place'],book_title=item['container-title'],series=item['collection-title'],doi=item['DOI'],keywords=keywords,authors=self._author(item['author']),source=src)
    
    def _asJournal(self,item,src):
        keywords = []
        if "keyword" in item:
            keywords = item['keyword'].split(',')
        return Publication.Journal(peer_reviewed=True,publisher="ACM",title=item['title'],source=src,year=item['original-date']['date-parts'][0][0],authors=self._author(item['author']),country=None,journal_name=item['container-title'],volume=item['volume'],num_pages=item['number-of-pages'],doi=item['DOI'],keywords=keywords)

    
# Create a function called "chunks" with two arguments, l and n:
def chunks(l, n):
    # For item i in a range that is a length of l,
    for i in range(0, len(l), n):
        # Create an index range for l of n items:
        yield l[i:i+n]



from abc import ABCMeta,abstractmethod


class Finder(metaclass=ABCMeta):
    
    @abstractmethod
    def name(self,): raise NotImplementedError

    @abstractmethod
    def search(self,keywords=[]): raise NotImplementedError

    @abstractmethod
    def search_raw(self,query): raise NotImplementedError



import scholarly
from pyslr.find import Finder
from pyslr.common import Format

class ScholarSearch(Finder):
    def name():
        return "Google Scholar"


    @abstractmethod
    def search(keywords=[]):
        results = scholarly.search_pubs_query("' '".join(keywords))
        for result in results:
            result


    @abstractmethod
    def search_raw(query): raise NotImplementedError



from pyslr.importer import BibImporter
from pyslr.common import Format
import logging

if __name__ == "__main__":
    import sys
    print("pyslr")
    logging.basicConfig(level=logging.DEBUG)

    args = sys.argv[1:]
    bibfile = args[0]
    output = args[1]
    slr = Format()
    with open(output,"w") as out:
        writer = slr.openWriter(out)

        bibin = BibImporter()

        bibin.parse_and_write(bibfile,slr)


import logging
from datetime import datetime
import bibtexparser
from pylatexenc.latex2text import LatexNodes2Text
from pyslr.common import Format,Publication


logger = logging.getLogger(__name__)

__all__ = ["BibImporter"]

class BibImporter():

    def __init__(self):
        super().__init__()
        self.inproceedingsTypes = ["inproceedings"]
        self.articleTypes = ["article"]
        self.thesisTypes = ["phdthesis"]
        self.bookTypes = ["book"]
        self.validTypes = self.inproceedingsTypes+self.articleTypes+self.thesisTypes+self.bookTypes


    def open_file(self,filename):
        return open(filename,"r")

    def parse_and_write(self,reader,writer):

        with reader as bibtex_file:
            bib_database = bibtexparser.load(bibtex_file)

        inproceedings = filter(lambda x:x["ENTRYTYPE"] in self.inproceedingsTypes,bib_database.entries)
        
        ignored = filter(lambda x:x["ENTRYTYPE"] not in self.validTypes,bib_database.entries)

        for i in ignored:
            logger.warning("ignoring %s because of unknown type (%s)",i["ID"],i["ENTRYTYPE"])

        for inp in inproceedings:
            tried = ["id","peer_reviewed","publisher","title","year","authors","type","country","num_pages"]
            pub = Publication.Paper(
                        peer_reviewed=1,
                        publisher=self.get("publisher",inp,default="unknown"),
                        source="bib_import_v1",
                        title=self._get("title",inp,True),
                        year=self._get("year",inp,True),
                        authors=self.authors(inp),
                        country=self.read_country(inp),
                        book_title=self.get("booktitle",inp,default=None,tried=tried),
                        series=self.get("series",inp,default=None,tried=tried),
                        num_pages=self.pages(inp),
                        doi=self.optional("doi",inp,tried=tried),
                        keywords=self.keywords(inp,log_error=False),
                        abstract=None,
                        tried=tried,
                        references=None)
            writer.addRow(pub)
        
        article = filter(lambda x:x["ENTRYTYPE"]in self.articleTypes,bib_database.entries)

        for inp in article:
            tried = ["id","peer_reviewed","publisher","title","year","authors","type","num_pages"]
            pub = Publication.Journal(
                peer_reviewed=1,
                publisher=self.get("publisher",inp,default="unknown"),
                source='bib_import_v1',
                title=self._get("title",inp,True),
                year=self._get("year",inp,True),
                authors=self.authors(inp),
                country=self.read_country(inp,log_error=False),
                journal_name=self.get("journal",inp,default=None,tried=tried),
                volume=self.get("volume",inp,default=None,tried=tried),
                num_pages=self.pages(inp),
                doi=self.get("doi",inp,default=None,tried=tried),
                keywords=self.keywords(inp,log_error=False),
                abstract=None,
                tried=tried,
                references=None
            )
            writer.addRow(pub)
        
        
        thesis = filter(lambda x:x["ENTRYTYPE"]in self.thesisTypes,bib_database.entries)

        for inp in thesis:
            tried = ["id","peer_reviewed","publisher","title","year","authors","type",]
            pub = Publication.PHD(
                peer_reviewed=1,
                publisher=self.get("publisher",inp,default="unknown",log_error=False),
                source='bib_import_v1',
                title=self._get("title",inp,True),
                year=self._get("year",inp,True),
                authors=self.authors(inp),
                country=self.read_country(inp,log_error=False),
                school=self._get("school",inp,True),
                num_pages=self.pages(inp,log_error=False),
                doi=self.optional("doi",inp,tried=tried),
                abstract=None,
                isbn=self.optional("isbn",inp,tried=tried),
                references=None,
                tried=tried
            )
            writer.addRow(pub)

        
        book = filter(lambda x:x["ENTRYTYPE"] in self.bookTypes,bib_database.entries)

        for inp in book:
            tried = ["id","peer_reviewed","publisher","title","year","authors","type",]
            pub = Publication.Book(
                peer_reviewed=1,
                publisher=self.get("publisher",inp,default="unknown"),
                source='bib_import_v1',
                title=self._get("title",inp,True),
                year=self._get("year",inp,True),
                authors=self.authors(inp),
                num_pages=self.pages(inp,log_error=False),
                doi=self.optional("doi",inp,tried=tried),
                isbn=self.get("isbn",inp,default=None,tried=tried),
                references=None,
                tried=tried
            )
            
            writer.addRow(pub)


    def _get(self,key,bibentry,compile_latex):
        if key in bibentry:
            value =  bibentry[key]
            if compile_latex:
                return LatexNodes2Text().latex_to_text(value)
            return value
        else:
            raise KeyError(key)

    def get(self,key,bibentry,compile_latex=True,default="",tried=[],log_error=True):
        value = default
        tried.append(key)
        try:
            value = self._get(key,bibentry,compile_latex)
            return value
        except KeyError:
            if log_error:
                logger.warning("%s missing in %s",key,bibentry["ID"])
            return default

    def optional(self,key,bibentry,compile_latex=True,tried=[]):
        return self.get(key,bibentry,compile_latex,None,tried,False)
        

    def authors(self,bibentry,compile_latex=True):
        value = self._get("author",bibentry,compile_latex)

        return list(map(lambda x:"(%s)"%x.strip(),value.split("and")))

    def pages(self,bibentry,compile_latex=True,log_error=True):
        value = None
        if "numpages" in bibentry:
            value = bibentry["numpages"]

        #try to parese xx-yy format of pages and calulate them
        if "pages" in bibentry and value == "":
            value = bibentry["pages"]
            if "-" in value:
                pages = value.split("-")
                #remove empty cells due to (--)
                pages = list(filter(lambda x:len(x) > 0,pages))
                try:
                    value = str(int(pages[1])-int(pages[0]))
                except ValueError:
                    if log_error:
                        logger.warning("could not deterine pages of %s found only %s",bibentry["ID"],pages)

        return value

    def read_country(self,bibentry,compile_latex=True,log_error=True):
        try:
            value = self._get("address",bibentry,compile_latex)
            if "," in value:
                return value.split(",")[-1]
            else:
                return value
        except KeyError:
            if log_error:
                logger.warning("address missing in %s",bibentry["ID"])
            return None

    def keywords(self,bibentry,compile_latex=True,log_error=True):
        try:
            value = self._get("keywords",bibentry,compile_latex)
            return list(map(lambda x:"%s"%x.strip(),value.split(",")))
        except KeyError:
            if log_error:
                logger.warning("keywords missing in %s",bibentry["ID"])
            return None



#!/usr/bin/env python

from setuptools import setup
# import os 
# import re
# # The full version, including alpha/beta/rc tags.
# release = re.sub('^v', '', os.popen('git describe').read().strip())
# # The short X.Y version.
# version = release

setup(
    setup_requires=['pbr>=1.9', 'setuptools>=17.1'],
    pbr=True,
)